--
-- DE2 (Cyclone-II) Entity for Interactive Project Game
-- Authors:
--      Abdulhamid Ghandour
--      Thomas John
--      Jaime Peretzman
--      Bharadwaj Vellore
--
-- Desc:
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity soundcontroller is
  
  port (
    clk        : in  std_logic;
    reset_n    : in  std_logic;
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  unsigned(3 downto 0);
    readdata   : out unsigned(31 downto 0);
    writedata  : in  unsigned(31 downto 0);
    aud_xck    : out std_logic;
    aud_adclrck: out std_logic;
    aud_adcdat : in std_logic;
    aud_daclrck: out std_logic;
    aud_dacdat : out std_logic;
    aud_bclk   : inout std_logic
  );
end soundcontroller;

architecture rtl of soundcontroller is

  type ram_type is array(7 downto 0) of unsigned(31 downto 0);
  signal RAM : ram_type;
  signal ram_address : unsigned(2 downto 0);
  signal counter  : unsigned(31 downto 0);
  signal audio_clock : unsigned(1 downto 0) := "00";
  signal audio_request : std_logic;
  signal audio_ctrl: std_logic := '0';

  component de2_wm8731_audio port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    test_mode : in std_logic;       --    Audio CODEC controller test mode
    audio_request : out std_logic;  --    Audio controller request new data
    data : in unsigned(15 downto 0);
  
    -- Audio interface signals
    AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
    AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
  end  component;

begin
  ram_address <= address(2 downto 0);

  audio_clk_gen: process (clk)
  begin
    if rising_edge(clk) then
      audio_clock <= audio_clock + "1";
    end if;
  end process audio_clk_gen;

  audio_host_control: process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        
      else
        if chipselect = '1' then
          if read = '1' then
            readdata <= RAM(to_integer(ram_address));
          elsif write = '1' then
            RAM(to_integer(ram_address)) <= writedata;
          end if;
        end if;
-------------------------------------
		if audio_clock = "00" then
			if RAM(0)(0) = '1' then
				audio_ctrl <= '1';
				RAM(0)(0) <= '0';
			end if;
			
			if audio_ctrl = '1' then
				audio_ctrl <= '0';
			end if;
        end if;
--------------------------------------      
      end if;
    end if;
  end process audio_host_control;

--  audio_state_ctrl: process (clk)
--  begin
--    if rising_edge (clk) then
--    
----        counter <= (others => '0');
----      else
----        if counter = 100 then
----          counter <= (others => '0');
----          reset_ctrl <= '0';
----        else    
----          counter <= counter + 1;
----        end if;
--      end if;
--    end if;
--  end process audio_state_ctrl;

  aud_xck <= audio_clock(1);

  beeper: de2_wm8731_audio port map (
    clk => audio_clock(1),
    reset_n => '1',
    test_mode => audio_ctrl,               -- Output a sine wave
    audio_request => audio_request,
    data => "0000000000000000",
    AUD_ADCLRCK  => aud_adclrck,
    AUD_ADCDAT   => aud_adcdat,
    AUD_DACLRCK  => aud_daclrck,
    AUD_DACDAT   => aud_dacdat,
    AUD_BCLK     => aud_bclk
  );
end rtl;
